program Coll8Console;

{$APPTYPE CONSOLE}
{$R *.res}

uses
  System.SysUtils,
  ConnectionServer in 'ConnectionServer.pas' {ServerContainer: TDataModule},
  Coll8ServiceImplementation in 'Coll8ServiceImplementation.pas',
  Coll8Service in 'Coll8Service.pas',
  Sparkle.Indy.Server,
  IdStack,
  Sparkle.HttpServer.Module,
  Sparkle.HttpServer.Context,
  XData.OpenApi.Service.Internal,
  SMXi_Debug in '..\Shared\SMXi_Debug.pas',
  GFSProxyDtos in 'GFSProxyAPI\GFSProxyDtos.pas',
  smx.OpenAPI.Utils in 'smx.OpenAPI.Utils.pas';

begin
  var oServer: TIndySparkleHTTPServer := nil;
  var bQuit: Boolean := False;
  var sInput: string := '';
  var oModule: THttpServerModule;
  var iPort: Integer := StrToIntDef(GetEnvironmentVariable('PORT'), 2019);
  var sPort: string := IntToStr(iPort);

  try

    ServerContainer := TServerContainer.Create(nil);
    ServerContainer.XDataServer.BaseUrl := 'http://+:' + sPort + '/Coll8';

    oServer := TIndySparkleHTTPServer.Create(nil);

    try
      oModule := ServerContainer.XDataServer.CreateModule;
      oServer.Dispatcher.AddModule(oModule);

      // customize the swagger definitions
      oModule.AddMiddleware(
        TAnonymousMiddleware.Create(
          procedure(Context: THttpServerContext; Next: THttpServerProc)
          begin
            Context.SetItem<IOpenApiEvents>(TSMXiOpenApiEvents.Create);
            Next(Context);

          end
        )
      );

      // apiky validation
      oModule.AddMiddleware(
        TAnonymousMiddleware.Create(
          procedure(Context: THttpServerContext; Next: THttpServerProc)
          begin
             //ApiKy call really here
              Next(Context);

          end
        )
      );


      oServer.DefaultPort := iPort;

      {$IFDEF LINUX}
      oServer.Active := True;
      {$ELSE}
      oServer.StartListening;
      {$ENDIF}

      WriteLn('Server Started: ' + DateTimeToStr(Now));

      { TODO : We should have an environment var to let us publish the real server name/ip address rather than <server> or the + in BaseUrl }

      WriteLn('Listening on ' + ServerContainer.XDataServer.BaseUrl);

      WriteLn('Swagger UI: http://<server>:' + sPort + '/Coll8/swaggerui');

      WriteLn('Swagger: http://<server>:' + sPort + '/Coll8/openapi/swagger.json');

      WriteLn('Enter "stop" to quit');

      while not bQuit do
      begin
        ReadLn(sInput);
        bQuit := SameText(sInput, 'stop');
      end;

    except
      on E: Exception do
      begin
        Writeln(E.ClassName, ': ', E.Message);
      end;

    end;

  finally

    if Assigned(oServer) then
      oServer.Free;

    if Assigned(ServerContainer) then
      ServerContainer.Free;

  end;

end.
