unit GFSProxyDtos;

{$IFDEF FPC}{$MODE Delphi}{$ENDIF}

interface

uses
  Generics.Collections,
  SysUtils,
  Bcl.Json.Attributes,
  smx.OpenAPI.Utils,
  System.JSON;

type
  TLabelSpecificationClass = class;
  TWeightClass = class;
  TConsignmentClass = class;
  TDimensionsClass = class;
  TPackageClass = class;
  TGFSPackageList = class;
  TCustomsDataCommodityValueClass = class;
  TCustomsDataCommodityClass = class;
  TCustomsDataCommoditiesItemList = class;
  TCustomsDataClass = class;
  TAddressClass = class;
  TServiceOptionsClass = class;
  TCreateLabelRequestClass = class;
  TCreateLabelResponsePackageItemClass = class;
  TCreateLabelResponsePackagesItemList = class;
  TCreateLabelResponseClass = class;
  TError = class;
  TGFSReturnShipment = class;

  TLabelSpecificationClass = class
  private
    FlabelFormat: string;
  public
    property labelFormat: string read FlabelFormat write FlabelFormat;
  end;

  TWeightClass = class
  private
    [JsonProperty('unit')]
    /// <swagger name="required" />
    FunitOfWeight: string;
    /// <swagger name="required" />
    Fvalue: Double;
  public
    property unitOfWeight: string read FunitOfWeight write FunitOfWeight;
    property value: Double read Fvalue write Fvalue;
  end;
  
  TConsignmentClass = class
  private
    /// <swagger name="required" />
    Fweight: TWeightClass;
    procedure SetWeight(const Value: TWeightClass);
  public
    constructor Create;
    destructor Destroy; override;
    property weight: TWeightClass read Fweight write SetWeight;
  end;
  
  TDimensionsClass = class
  private
    /// <swagger name="required" />
    Flength: Double;
    /// <swagger name="required" />
    Fwidth: Double;
    /// <swagger name="required" />
    Fheight: Double;
    [JsonProperty('unit')]
    /// <swagger name="required" />
    FDimensionUnit: string;
  public
    property length: Double read Flength write Flength;
    property width: Double read Fwidth write Fwidth;
    property height: Double read Fheight write Fheight;
    property DimensionUnit: string read FDimensionUnit write FDimensionUnit;
  end;
  
  TPackageClass = class
  private
    Fdescription: string;
    [JsonIgnore]
    FdescriptionHasValue: Boolean;
    FinvoiceNumber: string;
    [JsonIgnore]
    FinvoiceNumberHasValue: Boolean;
    /// <swagger name="required" />
    Fdimensions: TDimensionsClass;
    FpackageClientReferenceId: string;
    FpackageType: string;
    procedure SetDescription(const Value: string);
    procedure SetInvoiceNumber(const Value: string);
    procedure SetDimensions(const Value: TDimensionsClass);
  public
  	constructor Create;
    destructor Destroy; override;
    property description: string read Fdescription write SetDescription;
    property descriptionHasValue: Boolean read FdescriptionHasValue write FdescriptionHasValue;
    /// <summary>
    /// Invoice number. Required for international deliveries
    /// </summary>
    property invoiceNumber: string read FinvoiceNumber write SetInvoiceNumber;
    property invoiceNumberHasValue: Boolean read FinvoiceNumberHasValue write FinvoiceNumberHasValue;
    property dimensions: TDimensionsClass read Fdimensions write SetDimensions;
    property packageClientReferenceId: string read FpackageClientReferenceId write FpackageClientReferenceId;
    /// <summary>
    ///   This is additional to DPD
    /// </summary>
    property packagingType: string read FpackageType write FpackageType;
  end;

  TGFSPackageList = class(TObjectList<TPackageClass>)
  end;

  TCustomsDataCommodityValueClass = class
  private
  /// <swagger name="required" />
    Fcurrency: string;
    /// <swagger name="required" />
    Fvalue: Double;
  public
    property currency: string read Fcurrency write Fcurrency;
    property value: Double read FValue write FValue;
  end;

  
  TCustomsDataCommodityClass = class
  private
    Fquantity: Integer;
    Fdescription: string;
    FhsCode: string;
    /// <swagger name="required" />
    Fvalue: TCustomsDataCommodityValueClass;
    /// <swagger name="required" />
    Fweight: TWeightClass;
    FcountryOfOrigin: string;
    FpackagingType: string;
    procedure SetValue(const Value: TCustomsDataCommodityValueClass);
    procedure SetWeight(const Value: TWeightClass);
  public
    constructor Create;
    destructor Destroy; override;
    property quantity: Integer read Fquantity write Fquantity;
    property description: string read Fdescription write Fdescription;
    property hsCode: string read FhsCode write FhsCode;
    property value: TCustomsDataCommodityValueClass read Fvalue write SetValue;
    property weight: TWeightClass read Fweight write SetWeight;
    /// <summary>
    ///   This is additional to DPD
    /// </summary>
    property countryOfOrigin: string read FcountryOfOrigin write FcountryOfOrigin;
    /// <summary>
    ///   This is additional to DPD
    /// </summary>
    property packagingType: string read FpackagingType write FpackagingType;
  end;

  TCustomsDataCommoditiesItemList = class(TObjectList<TCustomsDataCommodityClass>)
  end;

  /// <summary>
  /// Only required for shipments to non-EU locations
  /// </summary>
  TCustomsDataClass = class
  private
    /// <swagger name="required" />
    Fcommodities: TCustomsDataCommoditiesItemList;
    procedure SetCommodities(const Value: TCustomsDataCommoditiesItemList);
  public
    constructor Create;
    destructor Destroy; override;
    property commodities: TCustomsDataCommoditiesItemList read Fcommodities write SetCommodities;
  end;

  TAddressClass = class
  private
    Fname: string;
    FcompanyName: string;
    [JsonIgnore]
    FcompanyNameHasValue: Boolean;
    FaddressLine1: string;
    FaddressLine2: string;
    [JsonIgnore]
    FaddressLine2HasValue: Boolean;
    Fstate: string;
    [JsonIgnore]
    FstateHasValue: Boolean;
    Fcity: string;
    FcountryCode: string;
    FpostalCode: string;
    Femail: string;
    [JsonIgnore]
    FemailHasValue: Boolean;
    FphoneNumber: string;
    [JsonIgnore]
    FphoneNumberHasValue: Boolean;
    Fvat: string;
    [JsonIgnore]
    FvatHasValue: Boolean;
    Feori: string;
    [JsonIgnore]
    FeoriHasValue: Boolean;
    FmobileNumber: string;
    procedure SetCompanyName(const Value: string);
    procedure SetAddressLine2(const Value: string);
    procedure SetState(const Value: string);
    procedure SetEmail(const Value: string);
    procedure SetPhoneNumber(const Value: string);
    procedure SetVat(const Value: string);
    procedure SetEori(const Value: string);
  public
    /// <summary>
    /// The name of the person or business at the address
    /// </summary>
    property name: string read Fname write Fname;
    /// <summary>
    /// The name of the company or business at the address
    /// </summary>
    property companyName: string read FcompanyName write SetCompanyName;
    property companyNameHasValue: Boolean read FcompanyNameHasValue write FcompanyNameHasValue;
    /// <summary>
    /// First line of the address
    /// </summary>
    property addressLine1: string read FaddressLine1 write FaddressLine1;
    /// <summary>
    /// Additional address information if required
    /// </summary>
    property addressLine2: string read FaddressLine2 write SetAddressLine2;
    property addressLine2HasValue: Boolean read FaddressLine2HasValue write FaddressLine2HasValue;
    /// <summary>
    /// State in ISO 3166-2 code (if applicable)
    /// </summary>
    property state: string read Fstate write SetState;
    property stateHasValue: Boolean read FstateHasValue write FstateHasValue;
    /// <summary>
    /// The city where the person or business is located
    /// </summary>
    property city: string read Fcity write Fcity;
    /// <summary>
    /// The ISO 3166-1 alpha-2 two digit country code where the person or business is located
    /// </summary>
    property countryCode: string read FcountryCode write FcountryCode;
    /// <summary>
    /// The postal code for an address
    /// </summary>
    property postalCode: string read FpostalCode write FpostalCode;
    /// <summary>
    /// The email address of the person or business at the address
    /// </summary>
    property email: string read Femail write SetEmail;
    property emailHasValue: Boolean read FemailHasValue write FemailHasValue;
    /// <summary>
    /// The phone number of the person or business at the address
    /// </summary>
    property phoneNumber: string read FphoneNumber write SetPhoneNumber;
    property phoneNumberHasValue: Boolean read FphoneNumberHasValue write FphoneNumberHasValue;
    /// <summary>
    ///   Additional to DPD. Required for ShipTo Addresses
    /// </summary>
    property mobileNumber: string read FmobileNumber write FmobileNumber;
    /// <summary>
    /// VAT number, either this or EORI is required for international deliveries
    /// </summary>
    property vat: string read Fvat write SetVat;
    property vatHasValue: Boolean read FvatHasValue write FvatHasValue;
    /// <summary>
    /// EORI number, either this or VAT is required for international deliveries
    /// </summary>
    property eori: string read Feori write SetEori;
    property eoriHasValue: Boolean read FeoriHasValue write FeoriHasValue;
  end;

  TServiceOptionsClass = class
  private
    FsaturdayDelivery: Boolean;
    [JsonIgnore]
    FsaturdayDeliveryHasValue: Boolean;
    procedure SetSaturdayDelivery(const Value: Boolean);
  public
    property saturdayDelivery: Boolean read FsaturdayDelivery write SetSaturdayDelivery;
    property saturdayDeliveryHasValue: Boolean read FsaturdayDeliveryHasValue write FsaturdayDeliveryHasValue;
  end;

  TCreateLabelRequestClass = class
  private
    /// <swagger name="required" />
    FserviceProvider: string;
    /// <swagger name="required" />
    FlabelSpecification: TLabelSpecificationClass;
    Fconsignment: TConsignmentClass;
    /// <swagger name="required" />
    Fpackages: TGFSPackageList;
    Fcustoms: TCustomsDataClass;
    /// <swagger name="required" />
    FshipFrom: TAddressClass;
    /// <swagger name="required" />
    FshipTo: TAddressClass;
    /// <swagger name="required" />
    FserviceType: string;
    /// <swagger name="required" />
    FserviceOptions: TServiceOptionsClass;
    procedure SetLabelSpecification(const Value: TLabelSpecificationClass);
    procedure SetConsignment(const Value: TConsignmentClass);
    procedure SetPackages(const Value: TGFSPackageList);
    procedure SetCustoms(const Value: TCustomsDataClass);
    procedure SetShipFrom(const Value: TAddressClass);
    procedure SetShipTo(const Value: TAddressClass);
    procedure SetServiceOptions(const Value: TServiceOptionsClass);
  public
    constructor Create;
    destructor Destroy; override;
    function numberOfPackages: Integer;
    property serviceProvider: string read FserviceProvider write FserviceProvider;
    property labelSpecification: TLabelSpecificationClass read FlabelSpecification write SetLabelSpecification;
    property consignment: TConsignmentClass read Fconsignment write SetConsignment;
    property packages: TGFSPackageList read Fpackages write SetPackages;
    property customs: TCustomsDataClass read Fcustoms write SetCustoms;
    property shipFrom: TAddressClass read FshipFrom write SetShipFrom;
    property shipTo: TAddressClass read FshipTo write SetShipTo;
    property serviceType: string read FserviceType write FserviceType;
    property serviceOptions: TServiceOptionsClass read FserviceOptions write SetServiceOptions;
  end;
  
  TCreateLabelResponsePackageItemClass = class
  private
    FDocuments: string;
    FTrackingId: string;
  public
    property Documents: string read FDocuments write FDocuments;
    property TrackingId: string read FTrackingId write FTrackingId;
  end;

  TCreateLabelResponsePackagesItemList = class(TObjectList<TCreateLabelResponsePackageItemClass>)
  end;

  TCreateLabelResponseClass = class
  private
    /// <swagger name="required" />
    FPackages: TCreateLabelResponsePackagesItemList;
    /// <swagger name="required" />
    FErrors: TObjectList<TErrorClass>;
    procedure SetPackages(const Value: TCreateLabelResponsePackagesItemList);
  public
    constructor Create;
    destructor Destroy; override;
    property Packages: TCreateLabelResponsePackagesItemList read FPackages write SetPackages;
    property Errors: TObjectList<TErrorClass> read FErrors write FErrors;
  end;
  
  TError = class
  private
    FCode: string;
    FMessage: string;
    FDetails: string;
    [JsonIgnore]
    FDetailsHasValue: Boolean;
    procedure SetDetails(const Value: string);
  public
    property Code: string read FCode write FCode;
    property Message: string read FMessage write FMessage;
    property Details: string read FDetails write SetDetails;
    property DetailsHasValue: Boolean read FDetailsHasValue write FDetailsHasValue;
  end;

  TEndOfDayManifestClass = class
  private
    FshippingDate: TDate;
    FtrackingIds: TsmxStringList;
  public
    destructor Destroy; override;
    property shippingDate: TDate read FshippingDate write FshippingDate;
    property trackingIds: TsmxStringList read FtrackingIds write FtrackingIds;
  end;


  TGFSRetailerInvoice = class
  private
    FinvoiceNumber: string;
    FcurrencyCode: string;
    FtotalAmount: Double;
    FshippingCost: Double;
    FshippingInsuranceCost: Double;
    FretailerOSSAccountCode: string;
    FdeliveryTerms: string;
    FtanAccountNumber: string;
    FinvoiceDate: string;
    FotherCharges: Double;
    FreasonForExport: string;
  public
    property invoiceNumber: string read FinvoiceNumber write FinvoiceNumber;
    property currencyCode: string read FcurrencyCode write FcurrencyCode;
    property totalAmount: Double read FtotalAmount write FtotalAmount;
    property shippingCost: Double read FshippingCost write FshippingCost;
    property shippingInsuranceCost: Double read FshippingInsuranceCost write FshippingInsuranceCost;
    property retailerOSSAccountCode: string read FretailerOSSAccountCode write FretailerOSSAccountCode;
    property deliveryTerms: string read FdeliveryTerms write FdeliveryTerms;
    property tanAccountNumber: string read FtanAccountNumber write FtanAccountNumber;
    property invoiceDate: string read FinvoiceDate write FinvoiceDate;
    property otherCharges: Double read FotherCharges write FotherCharges;
    property reasonForExport: string read FreasonForExport write FreasonForExport;
  end;


  TGFSReturnShipment = class
  private
    FRetailerId: Integer;
    [JsonIgnore]
    FRetailerIdHasValue: Boolean;
    FRetailer: string;
    [JsonIgnore]
    FRetailerHasValue: Boolean;
    FOrderReference: string;
    [JsonIgnore]
    FOrderReferenceHasValue: Boolean;
    FShipFromAddress: TAddressClass;
    FNumberOfPackages: Integer;
    [JsonIgnore]
    FNumberOfPackagesHasValue: Boolean;
    FPackageItems: TCustomsDataCommoditiesItemList;
    FRetailerInvoice: TGFSRetailerInvoice;
    FCustomerReference: string;
    [JsonIgnore]
    FCustomerReferenceHasValue: Boolean;
  public
    destructor Destroy; override;
    property RetailerId: Integer read FRetailerId write FRetailerId;
    property RetailerIdHasValue: Boolean read FRetailerIdHasValue write FRetailerIdHasValue;
    property Retailer: string read FRetailer write FRetailer;
    property RetailerHasValue: Boolean read FRetailerHasValue write FRetailerHasValue;
    property OrderReference: string read FOrderReference write FOrderReference;
    property OrderReferenceHasValue: Boolean read FOrderReferenceHasValue write FOrderReferenceHasValue;
    property ShipFromAddress: TAddressClass read FShipFromAddress write FShipFromAddress;
    property NumberOfPackages: Integer read FNumberOfPackages write FNumberOfPackages;
    property NumberOfPackagesHasValue: Boolean read FNumberOfPackagesHasValue write FNumberOfPackagesHasValue;
    property PackageItems: TCustomsDataCommoditiesItemList read FPackageItems write FPackageItems;
    property RetailerInvoice: TGFSRetailerInvoice read FRetailerInvoice write FRetailerInvoice;
    property CustomerReference: string read FCustomerReference write FCustomerReference;
    property CustomerReferenceHasValue: Boolean read FCustomerReferenceHasValue write FCustomerReferenceHasValue;
  end;
  
implementation

{ TConsignmentClass }

constructor TConsignmentClass.Create;
begin
  inherited;
  Fweight := TWeightClass.Create;
end;

destructor TConsignmentClass.Destroy;
begin
  Fweight.Free;
  inherited;
end;

procedure TConsignmentClass.SetWeight(const Value: TWeightClass);
begin
  if Value <> Fweight then
  begin
    Fweight.Free;
    Fweight := Value;
  end;
end;

{ TPackageClass }

constructor TPackageClass.Create;
begin
  inherited;
  Fdimensions := TDimensionsClass.Create;

end;

destructor TPackageClass.Destroy;
begin
  Fdimensions.Free;
  inherited;
end;

procedure TPackageClass.SetDescription(const Value: string);
begin
  Fdescription := Value;
  FdescriptionHasValue := True;
end;

procedure TPackageClass.SetInvoiceNumber(const Value: string);
begin
  FinvoiceNumber := Value;
  FinvoiceNumberHasValue := True;
end;

procedure TPackageClass.SetDimensions(const Value: TDimensionsClass);
begin
  if Value <> Fdimensions then
  begin
    Fdimensions.Free;
    Fdimensions := Value;
  end;
end;

{ TCustomsDataCommodityClass }

constructor TCustomsDataCommodityClass.Create;
begin
  inherited;
  Fvalue := TCustomsDataCommodityValueClass.Create;
  Fweight := TWeightClass.Create;
end;

destructor TCustomsDataCommodityClass.Destroy;
begin
  Fweight.Free;
  Fvalue.Free;
  inherited;
end;

procedure TCustomsDataCommodityClass.SetValue(const Value: TCustomsDataCommodityValueClass);
begin
  if Value <> Fvalue then
  begin
    Fvalue.Free;
    Fvalue := Value;
  end;
end;

procedure TCustomsDataCommodityClass.SetWeight(const Value: TWeightClass);
begin
  if Value <> Fweight then
  begin
    Fweight.Free;
    Fweight := Value;
  end;
end;

{ TCustomsData }

constructor TCustomsDataClass.Create;
begin
  inherited;
  Fcommodities := TCustomsDataCommoditiesItemList.Create;
end;

destructor TCustomsDataClass.Destroy;
begin
  Fcommodities.Free;
  inherited;
end;

procedure TCustomsDataClass.SetCommodities(const Value: TCustomsDataCommoditiesItemList);
begin
  if Value <> Fcommodities then
  begin
    Fcommodities.Free;
    Fcommodities := Value;
  end;
end;

{ TAddressClass }

procedure TAddressClass.SetCompanyName(const Value: string);
begin
  FcompanyName := Value;
  FcompanyNameHasValue := True;
end;

procedure TAddressClass.SetAddressLine2(const Value: string);
begin
  FaddressLine2 := Value;
  FaddressLine2HasValue := True;
end;

procedure TAddressClass.SetState(const Value: string);
begin
  Fstate := Value;
  FstateHasValue := True;
end;

procedure TAddressClass.SetEmail(const Value: string);
begin
  Femail := Value;
  FemailHasValue := True;
end;

procedure TAddressClass.SetPhoneNumber(const Value: string);
begin
  FphoneNumber := Value;
  FphoneNumberHasValue := True;
end;

procedure TAddressClass.SetVat(const Value: string);
begin
  Fvat := Value;
  FvatHasValue := True;
end;

procedure TAddressClass.SetEori(const Value: string);
begin
  Feori := Value;
  FeoriHasValue := True;
end;

{ TServiceOptionsClass }

procedure TServiceOptionsClass.SetSaturdayDelivery(const Value: Boolean);
begin
  FsaturdayDelivery := Value;
  FsaturdayDeliveryHasValue := True;
end;

{ TCreateLabelRequest }

constructor TCreateLabelRequestClass.Create;
begin
  inherited;
  FlabelSpecification := TLabelSpecificationClass.Create;
  Fconsignment := TConsignmentClass.Create;
  Fpackages := TGFSPackageList.Create;
  Fcustoms := TCustomsDataClass.Create;
  FshipFrom := TAddressClass.Create;
  FshipTo := TAddressClass.Create;
  FserviceOptions := TServiceOptionsClass.Create;

end;

destructor TCreateLabelRequestClass.Destroy;
begin
  FserviceOptions.Free;
  FshipTo.Free;
  FshipFrom.Free;
  Fcustoms.Free;
  Fpackages.Free;
  Fconsignment.Free;
  FlabelSpecification.Free;
  inherited;
end;

function TCreateLabelRequestClass.numberOfPackages: Integer;
begin
  if Assigned(Fpackages) then
     Result := Fpackages.Count
  else
     Result := 0;
end;

procedure TCreateLabelRequestClass.SetLabelSpecification(const Value: TLabelSpecificationClass);
begin
  if Value <> FlabelSpecification then
  begin
    FlabelSpecification.Free;
    FlabelSpecification := Value;
  end;
end;

procedure TCreateLabelRequestClass.SetConsignment(const Value: TConsignmentClass);
begin
  if Value <> Fconsignment then
  begin
    Fconsignment.Free;
    Fconsignment := Value;
  end;
end;

procedure TCreateLabelRequestClass.SetPackages(const Value: TGFSPackageList);
begin
  if Value <> Fpackages then
  begin
    Fpackages.Free;
    Fpackages := Value;
  end;
end;

procedure TCreateLabelRequestClass.SetCustoms(const Value: TCustomsDataClass);
begin
  if Value <> Fcustoms then
  begin
    Fcustoms.Free;
    Fcustoms := Value;
  end;
end;

procedure TCreateLabelRequestClass.SetShipFrom(const Value: TAddressClass);
begin
  if Value <> FshipFrom then
  begin
    FshipFrom.Free;
    FshipFrom := Value;
  end;
end;

procedure TCreateLabelRequestClass.SetShipTo(const Value: TAddressClass);
begin
  if Value <> FshipTo then
  begin
    FshipTo.Free;
    FshipTo := Value;
  end;
end;

procedure TCreateLabelRequestClass.SetServiceOptions(const Value: TServiceOptionsClass);
begin
  if Value <> FserviceOptions then
  begin
    FserviceOptions.Free;
    FserviceOptions := Value;
  end;
end;

{ TCreateLabelResponseClass }

constructor TCreateLabelResponseClass.Create;
begin
  inherited;
  FPackages := TCreateLabelResponsePackagesItemList.Create;
end;

destructor TCreateLabelResponseClass.Destroy;
begin
  FPackages.Free;
  inherited;
end;

procedure TCreateLabelResponseClass.SetPackages(const Value: TCreateLabelResponsePackagesItemList);
begin
  if Value <> FPackages then
  begin
    FPackages.Free;
    FPackages := Value;
  end;
end;

{ TError }

procedure TError.SetDetails(const Value: string);
begin
  FDetails := Value;
  FDetailsHasValue := True;
end;

{ TEndOfDayManifestClass }

destructor TEndOfDayManifestClass.Destroy;
begin
  if Assigned(FtrackingIds) then
     FtrackingIds.Free;
  inherited;
end;

{ TGFSReturnShipment }

destructor TGFSReturnShipment.Destroy;
begin
  if Assigned(FRetailerInvoice) then
     FRetailerInvoice.Free;
  if Assigned(FPackageItems) then
     FPackageItems.Free;
  if Assigned(FShipFromAddress) then
     FShipFromAddress.Free;
  inherited;
end;

end.
