unit smx.OpenAPI.Utils;

interface

uses
  System.Classes,
  System.Generics.Collections,
  OpenAPI.Document,
  OpenAPI.Classes.Operation,
  OpenAPI.Classes.Path,
  XData.OpenAPI.Service.Internal,
  Bcl.Json.Converters,
  XData.Model.Classes;

type

  TsmxStringList = class(TList<string>)
  public
    procedure Assign(Source: TsmxStringList);
  end;

  TErrorClass = class
  private
    FErrMessage: string;
    FPropertyName: string;
  public
    constructor Create(APropertyName, AErrMessage: string);
    function Summary: string;
    property PropertyName: string read FPropertyName write FPropertyName;
    property ErrMessage: string read FErrMessage write FErrMessage;
  end;

  TSMXiOpenApiEvents = class(TInterfacedObject, IOpenApiEvents)
  public
    procedure DocumentCreated(Document: TOpenApiDocument);
    procedure GetDefinitionName(var DefinitionName: string; TypeToken: TTypeToken);
    procedure ActionProcessed(Action: TXDataAction; Operation: TOperation; Path: TPathItem);

  end;

function ErrorsToList(Errors: TObjectList<TErrorClass>): string;

implementation

uses SysUtils;

function ErrorsToList(Errors: TObjectList<TErrorClass>): string;
var
  ts: TStringlist;
  error: TErrorClass;
begin
  ts := TStringlist.Create;
  try
    for error in Errors do
      ts.Add(error.Summary);
    result := ts.Text;
  finally
    ts.Free;
  end;
end;

constructor TErrorClass.Create(APropertyName, AErrMessage: string);
begin
  FPropertyName := APropertyName;
  FErrMessage := AErrMessage;
end;

{ TsmxStringList }

procedure TsmxStringList.Assign(Source: TsmxStringList);
var
  I: Integer;
begin
  for I := 0 to Source.Count - 1 do
    self.Add(Source[I]);
end;

function TErrorClass.Summary: string;
begin
  result := FPropertyName + ': ' + FErrMessage;
end;

{ TSMXiOpenApiEvents }

procedure TSMXiOpenApiEvents.ActionProcessed(Action: TXDataAction; Operation: TOperation; Path: TPathItem);
begin
end;

procedure TSMXiOpenApiEvents.DocumentCreated(Document: TOpenApiDocument);
begin
end;

procedure TSMXiOpenApiEvents.GetDefinitionName(var DefinitionName: string; TypeToken: TTypeToken);
var
  sDefinition: string;
  sChar: string;

begin
  // use '.' as namespace separators and remove all but the last entry
  sDefinition := DefinitionName;
  while Pos('.', sDefinition) <> 0 do
    System.Delete(sDefinition, 1, Pos('.', sDefinition));

  // remove 'class' from end of definition
  if Pos('class', LowerCase(sDefinition)) = (Length(sDefinition) - Length('class') + 1) then
    System.Delete(sDefinition, Length(sDefinition) - Length('class') + 1, Length('class'));

  // remove default 'T' from Delphi class and lowercase next letter
  if sDefinition[1] = 'T' then
    System.Delete(sDefinition, 1, 1);
  sChar := Copy(sDefinition, 1, 1);
  sChar := LowerCase(sChar);
  if (Length(sChar) > 0) and (Length(sDefinition) > 0) then
    sDefinition[1] := sChar[1];

  DefinitionName := sDefinition;

end;

end.
